PROGRAM DrawAdmap3Tape;

{$A+,B-,D+,E+,F-,I+,L+,N-,O-,R-,S-,V-}
{$M 16384,0,655360}

uses Graph, Crt;

PROCEDURE DisplayEntryMessage;
begin
  Writeln;
  Writeln ('F. MRAVENEC  -  An Interactive PCB Design System');
  Writeln ('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~');
  Writeln ('Program RdAdmap3, Version 3.20');
  Writeln ('A simple screen simulation of Admap 3 control tape');
  Writeln ('Copyright (C) 1989, 1991 by Petr Horsky');
  Writeln;
  if (ParamCount < 1) or (ParamCount > 2) then begin
    Writeln ('Usage:   DrAdmap3 <side A file> [<side B file>]');
    Writeln;
    Halt (2)
  end
end;

(* ADMAP CONTROL FORMAT CODES --------------------------------------------- *)

CONST
  FeedHole            = $00;
  RightOnlyCode       = $01;
  LeftOnlyCode        = $02;
  UpOnlyCode          = $04;
  RightUpCode         = $05;
  LeftUpCode          = $06;
  NextPatternCode     = $07;
  DownOnlyCode        = $08;
  RightDownCode       = $09;
  LeftDownCode        = $0A;
{ SmallPadCode        = $0B;   1 mm circle by the tool center}
{ LargePadCode        = $0C;   3 mm circle by the tool center}
  PreviousPatternCode = $0E;
  DrillHoleCode       = $0F;
  HomeCode            = $10;
  LightOnCode         = $11;
  SmallStepCode       = $13;   {0.25 (0.254) mm}
  LightOffCode        = $18;
  LargeStepCode       = $1C;   {2.5  (2.54)  mm}
  RuboutCode          = $1F;

VAR
  AdmapTape: file of byte;
  AdmapFileName: string;

(* INTERFACE TO ADMAP FILES, ADMAP 3 OUTPUT SIMULATION -------------------- *)

CONST
  LineColorA = 4;   {red}
  LineColorB = 3;   {cyan}
  MoveColor  = 8;   {dark gray}
  StepFactor = 1.0;

VAR
  AdmapX, AdmapY, CurrentStepSize, PathLength: real;
  LineColor: integer;

procedure OpenAdmap;
begin
  SetColor (MoveColor);
  SetWriteMode (XorPut);
  AdmapX := 0;   {CRT Coordinates}
  AdmapY := 0;
  PathLength := 0
end;

procedure OutputToAdmap (CommandCode: byte);
  var i: integer;
      ModifiedCommandCode: byte;
begin
  case CommandCode of
    FeedHole:       ;
    RightOnlyCode:  begin
                      Line (Round (AdmapX), Round (AdmapY),
                      Round (AdmapX + CurrentStepSize), Round (AdmapY));
                      AdmapX := AdmapX + CurrentStepSize;
                      PathLength := PathLength + CurrentStepSize
                    end;
    LeftOnlyCode:   begin
                      Line (Round (AdmapX), Round (AdmapY),
                            Round (AdmapX - CurrentStepSize), Round (AdmapY));
                      AdmapX := AdmapX - CurrentStepSize;
                      PathLength := PathLength + CurrentStepSize
                    end;
    UpOnlyCode:     begin
                      Line (Round (AdmapX), Round (AdmapY),
                            Round (AdmapX), Round (AdmapY -{+} CurrentStepSize));
                      AdmapY := AdmapY -{+} CurrentStepSize;
                      PathLength := PathLength + CurrentStepSize
                    end;
    RightUpCode:    begin
                      Line (Round (AdmapX), Round (AdmapY),
                            Round (AdmapX + CurrentStepSize), Round (AdmapY -{+} CurrentStepSize));
                      AdmapX := AdmapX + CurrentStepSize;
                      AdmapY := AdmapY -{+} CurrentStepSize;
                      PathLength := PathLength + CurrentStepSize
                    end;
    LeftUpCode:     begin
                      Line (Round (AdmapX), Round (AdmapY),
                            Round (AdmapX - CurrentStepSize), Round (AdmapY -{+} CurrentStepSize));
                      AdmapX := AdmapX - CurrentStepSize;
                      AdmapY := AdmapY -{+} CurrentStepSize;
                      PathLength := PathLength + CurrentStepSize
                    end;
    DownOnlyCode:   begin
                      Line (Round (AdmapX), Round (AdmapY),
                            Round (AdmapX), Round (AdmapY +{-} CurrentStepSize));
                      AdmapY := AdmapY +{-} CurrentStepSize;
                      PathLength := PathLength + CurrentStepSize
                    end;
    RightDownCode:  begin
                      Line (Round (AdmapX), Round (AdmapY),
                            Round (AdmapX + CurrentStepSize), Round (AdmapY +{-} CurrentStepSize));
                      AdmapX := AdmapX + CurrentStepSize;
                      AdmapY := AdmapY +{-} CurrentStepSize;
                      PathLength := PathLength + CurrentStepSize
                    end;
    LeftDownCode:   begin
                      Line (Round (AdmapX), Round (AdmapY),
                            Round (AdmapX - CurrentStepSize), Round (AdmapY +{-} CurrentStepSize));
                      AdmapX := AdmapX - CurrentStepSize;
                      AdmapY := AdmapY +{-} CurrentStepSize;
                      PathLength := PathLength + CurrentStepSize
                    end;
{
    NextPatternCode:     CurrentPatternNumber := (CurrentPatternNumber + 1) mod 100;
    PreviousPatternCode: CurrentPatternNumber := (CurrentPatternNumber + 99) mod 100;
}
    DrillHoleCode:  ;
    HomeCode:       ;

    LightOnCode:    begin
                      SetColor (LineColor);
                      SetWriteMode (OrPut);
                      Line (Round (AdmapX), Round (AdmapY), Round (AdmapX), Round (AdmapY))
                    end;
    LightOffCode:   begin
                      SetColor (MoveColor);
                      SetWriteMode (XorPut);
                    end;

    SmallStepCode:  CurrentStepSize := 1 * StepFactor;
    LargeStepCode:  CurrentStepSize := 10 * StepFactor;
    else            ;
  end {case}
end;

procedure CloseAdmap;
  var c: char;
begin
  Write (^G);
  repeat until KeyPressed;   c := ReadKey;
end;

(* LOW-LEVEL TAPE INTERFACE ----------------------------------------------- *)

const ByteReady: Boolean = false;

var   SavedByte: byte;

function ByteFromTape: byte;
  var b: byte;
begin
  if ByteReady then begin
    ByteFromTape := SavedByte;
    ByteReady := false
  end
  else begin
    repeat   {read two bytes from tape}
      if not Eof (AdmapTape) then
        Read (AdmapTape, b)
      else
        b := FeedHole;
      if not Eof (AdmapTape) then
        Read (AdmapTape, SavedByte)
      else
        SavedByte := FeedHole;
      if SavedByte <> RuboutCode then ByteReady := true
    until ByteReady;
    ByteFromTape := b
  end
end;

(* ------------------------------------------------------------------------ *)

{shadow} procedure EgaVgaDriver;     external;   {$L EgaVga.Obj}

{shadow} procedure HerculesDriver;   external;   {$L Herc.Obj}

var Driver, Mode, Dummy: integer;

BEGIN
  DisplayEntryMessage;
  DirectVideo := false;

  Dummy := RegisterBGIDriver (@EgaVgaDriver);
  Dummy := RegisterBGIDriver (@HerculesDriver);
  Driver := Detect;
  InitGraph (Driver, Mode, '');

  Assign (AdmapTape, ParamStr (1));
  Reset (AdmapTape);
  LineColor := LineColorA;
  OpenAdmap;
  while not Eof (AdmapTape) do OutputToAdmap (ByteFromTape);
  CloseAdmap;
  Close (AdmapTape);

  if ParamCount > 1 then begin
    Assign (AdmapTape, ParamStr (2));
    Reset (AdmapTape);
    LineColor := LineColorB;
    OpenAdmap;
    while not Eof (AdmapTape) do OutputToAdmap (ByteFromTape);
    CloseAdmap;
    Close (AdmapTape);
  end;

  CloseGraph;
END.
